@@ -14,9 +14,16 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define ALL_HOSTS                 "FF02::1"
+#define ALL_NODES                 "FF02::1"
 #define ALL_ROUTERS               "FF02::2"
 
+struct ping_packet {
+  u8 type, code;
+  u16 checksum;
+  u16 identifier;
+  u16 sequence_no;
+};
+
 struct ra_packet {
   u8 type, code;
   u16 checksum;
@@ -32,13 +39,11 @@ struct prefix_opt {
   struct in6_addr prefix;
 };
 
-#define ICMP6_ROUTER_SOLICIT 133
-#define ICMP6_ROUTER_ADVERT  134
-
 #define ICMP6_OPT_SOURCE_MAC   1
 #define ICMP6_OPT_PREFIX       3
 #define ICMP6_OPT_MTU          5
 #define ICMP6_OPT_RDNSS       25
+#define ICMP6_OPT_DNSSL       31
 
 
 
